@@ -16,17 +16,20 @@ plugin-specific operations.
 from collections.abc import Mapping
 from logging import LoggerAdapter
 import os
+import pathlib
 import shutil
 from tempfile import NamedTemporaryFile
-from typing import Any, List, Optional, Mapping as MappingType, Tuple, Union
+from typing import Any, List
+from typing import Mapping as MappingType
+from typing import Optional, Tuple, Union
 
 from aiida.common import AIIDA_LOGGER, exceptions
 from aiida.common.datastructures import CalcInfo
 from aiida.common.folders import SandboxFolder
 from aiida.common.links import LinkType
-from aiida.orm import load_node, CalcJobNode, Code, FolderData, Node, RemoteData
+from aiida.orm import CalcJobNode, Code, FolderData, Node, RemoteData, load_node
 from aiida.orm.utils.log import get_dblogger_extra
-from aiida.plugins import DataFactory
+from aiida.repository.common import FileType
 from aiida.schedulers.datastructures import JobState
 from aiida.transports import Transport
 
@@ -76,7 +79,7 @@ def upload_calculation(
     # before, which can happen if the daemon is restarted and it shuts down after uploading but before getting the
     # chance to perform the state transition. Upon reloading this calculation, it will re-attempt the upload.
     link_label = 'remote_folder'
-    if node.get_outgoing(RemoteData, link_label_filter=link_label).first():
+    if node.base.links.get_outgoing(RemoteData, link_label_filter=link_label).first():
         EXEC_LOGGER.warning(f'CalcJobNode<{node.pk}> already has a `{link_label}` output: skipping upload')
         return calc_info
 
@@ -89,10 +92,10 @@ def upload_calculation(
     transport.set_logger_extra(logger_extra)
     logger = LoggerAdapter(logger=EXEC_LOGGER, extra=logger_extra)
 
-    if not dry_run and node.has_cached_links():
+    if not dry_run and not node.is_stored:
         raise ValueError(
-            'Cannot submit calculation {} because it has cached input links! If you just want to test the '
-            'submission, set `metadata.dry_run` to True in the inputs.'.format(node.pk)
+            f'Cannot submit calculation {node.pk} because it is not stored! If you just want to test the submission, '
+            'set `metadata.dry_run` to True in the inputs.'
         )
 
     # If we are performing a dry-run, the working directory should actually be a local folder that should already exist
@@ -172,17 +175,17 @@ def upload_calculation(
     for code in input_codes:
         if code.is_local():
             # Note: this will possibly overwrite files
-            for filename in code.list_object_names():
+            for filename in code.base.repository.list_object_names():
                 # Note, once #2579 is implemented, use the `node.open` method instead of the named temporary file in
                 # combination with the new `Transport.put_object_from_filelike`
                 # Since the content of the node could potentially be binary, we read the raw bytes and pass them on
                 with NamedTemporaryFile(mode='wb+') as handle:
-                    handle.write(code.get_object_content(filename, mode='rb'))
+                    handle.write(code.base.repository.get_object_content(filename, mode='rb'))
                     handle.flush()
                     transport.put(handle.name, filename)
             transport.chmod(code.get_local_executable(), 0o755)  # rwxr-xr-x
 
-    # local_copy_list is a list of tuples, each with (uuid, dest_rel_path)
+    # local_copy_list is a list of tuples, each with (uuid, dest_path, rel_path)
     # NOTE: validation of these lists are done inside calculation.presubmit()
     local_copy_list = calc_info.local_copy_list or []
     remote_copy_list = calc_info.remote_copy_list or []
@@ -200,13 +203,26 @@ def upload_calculation(
         if data_node is None:
             logger.warning(f'failed to load Node<{uuid}> specified in the `local_copy_list`')
         else:
-            dirname = os.path.dirname(target)
-            if dirname:
-                os.makedirs(os.path.join(folder.abspath, dirname), exist_ok=True)
-            with folder.open(target, 'wb') as handle:
-                with data_node.open(filename, 'rb') as source:
-                    shutil.copyfileobj(source, handle)
-            provenance_exclude_list.append(target)
+
+            # If no explicit source filename is defined, we assume the top-level directory
+            filename_source = filename or '.'
+            filename_target = target or ''
+
+            # Make the target filepath absolute and create any intermediate directories if they don't yet exist
+            filepath_target = pathlib.Path(folder.abspath) / filename_target
+            filepath_target.parent.mkdir(parents=True, exist_ok=True)
+
+            if data_node.base.repository.get_object(filename_source).file_type == FileType.DIRECTORY:
+                # If the source object is a directory, we copy its entire contents
+                data_node.base.repository.copy_tree(filepath_target, filename_source)
+                provenance_exclude_list.extend(data_node.base.repository.list_object_names(filename_source))
+            else:
+                # Otherwise, simply copy the file
+                with folder.open(target, 'wb') as handle:
+                    with data_node.base.repository.open(filename, 'rb') as source:
+                        shutil.copyfileobj(source, handle)
+
+                provenance_exclude_list.append(target)
 
     # In a dry_run, the working directory is the raw input folder, which will already contain these resources
     if not dry_run:
@@ -257,7 +273,8 @@ def upload_calculation(
     else:
 
         if remote_copy_list:
-            with open(os.path.join(workdir, '_aiida_remote_copy_list.txt'), 'w') as handle:
+            filepath = os.path.join(workdir, '_aiida_remote_copy_list.txt')
+            with open(filepath, 'w', encoding='utf-8') as handle:  # type: ignore[assignment]
                 for remote_computer_uuid, remote_abs_path, dest_rel_path in remote_copy_list:
                     handle.write(
                         'would have copied {} to {} in working directory on remote {}'.format(
@@ -266,7 +283,8 @@ def upload_calculation(
                     )
 
         if remote_symlink_list:
-            with open(os.path.join(workdir, '_aiida_remote_symlink_list.txt'), 'w') as handle:
+            filepath = os.path.join(workdir, '_aiida_remote_symlink_list.txt')
+            with open(filepath, 'w', encoding='utf-8') as handle:  # type: ignore[assignment]
                 for remote_computer_uuid, remote_abs_path, dest_rel_path in remote_symlink_list:
                     handle.write(
                         'would have created symlinks from {} to {} in working directory on remote {}'.format(
@@ -289,9 +307,25 @@ def upload_calculation(
         for filename in filenames:
             filepath = os.path.join(root, filename)
             relpath = os.path.normpath(os.path.relpath(filepath, folder.abspath))
-            if relpath not in provenance_exclude_list:
-                with open(filepath, 'rb') as handle:
-                    node._repository.put_object_from_filelike(handle, relpath, 'wb', force=True)  # pylint: disable=protected-access
+            dirname = os.path.dirname(relpath)
+
+            # Construct a list of all (partial) filepaths
+            # For example, if `relpath == 'some/sub/directory/file.txt'` then the list of relative directory paths is
+            # ['some', 'some/sub', 'some/sub/directory']
+            # This is necessary, because if any of these paths is in the `provenance_exclude_list` the file should not
+            # be copied over.
+            components = dirname.split(os.sep)
+            dirnames = [os.path.join(*components[:i]) for i in range(1, len(components) + 1)]
+            if relpath not in provenance_exclude_list and all(
+                dirname not in provenance_exclude_list for dirname in dirnames
+            ):
+                with open(filepath, 'rb') as handle:  # type: ignore[assignment]
+                    node.base.repository._repository.put_object_from_filelike(handle, relpath)  # pylint: disable=protected-access
+
+    # Since the node is already stored, we cannot use the normal repository interface since it will raise a
+    # `ModificationNotAllowed` error. To bypass it, we go straight to the underlying repository instance to store the
+    # files, however, this means we have to manually update the node's repository metadata.
+    node.base.repository._update_repository_metadata()  # pylint: disable=protected-access
 
     if not dry_run:
         # Make sure that attaching the `remote_folder` with a link is the last thing we do. This gives the biggest
@@ -300,7 +334,7 @@ def upload_calculation(
         # task. Because in that case, the check for the existence of this link at the top of this function will exit
         # early from this command.
         remotedata = RemoteData(computer=computer, remote_path=workdir)
-        remotedata.add_incoming(node, link_type=LinkType.CREATE, link_label='remote_folder')
+        remotedata.base.links.add_incoming(node, link_type=LinkType.CREATE, link_label='remote_folder')
         remotedata.store()
 
 
@@ -388,7 +422,7 @@ def stash_calculation(calculation: CalcJobNode, transport: Transport) -> None:
         stash_mode=StashMode(stash_mode),
         source_list=source_list,
     ).store()
-    remote_stash.add_incoming(calculation, link_type=LinkType.CREATE, link_label='remote_stash')
+    remote_stash.base.links.add_incoming(calculation, link_type=LinkType.CREATE, link_label='remote_stash')
 
 
 def retrieve_calculation(calculation: CalcJobNode, transport: Transport, retrieved_temporary_folder: str) -> None:
@@ -412,7 +446,7 @@ def retrieve_calculation(calculation: CalcJobNode, transport: Transport, retriev
     # before, which can happen if the daemon is restarted and it shuts down after retrieving but before getting the
     # chance to perform the state transition. Upon reloading this calculation, it will re-attempt the retrieval.
     link_label = calculation.link_label_retrieved
-    if calculation.get_outgoing(FolderData, link_label_filter=link_label).first():
+    if calculation.base.links.get_outgoing(FolderData, link_label_filter=link_label).first():
         EXEC_LOGGER.warning(
             f'CalcJobNode<{calculation.pk}> already has a `{link_label}` output folder: skipping retrieval'
         )
@@ -427,17 +461,11 @@ def retrieve_calculation(calculation: CalcJobNode, transport: Transport, retriev
         # First, retrieve the files of folderdata
         retrieve_list = calculation.get_retrieve_list()
         retrieve_temporary_list = calculation.get_retrieve_temporary_list()
-        retrieve_singlefile_list = calculation.get_retrieve_singlefile_list()
 
         with SandboxFolder() as folder:
             retrieve_files_from_list(calculation, transport, folder.abspath, retrieve_list)
             # Here I retrieved everything; now I store them inside the calculation
-            retrieved_files.put_object_from_tree(folder.abspath)
-
-        # Second, retrieve the singlefiles, if any files were specified in the 'retrieve_temporary_list' key
-        if retrieve_singlefile_list:
-            with SandboxFolder() as folder:
-                _retrieve_singlefiles(calculation, transport, folder, retrieve_singlefile_list, logger_extra)
+            retrieved_files.base.repository.put_object_from_tree(folder.abspath)
 
         # Retrieve the temporary files in the retrieved_temporary_folder if any files were
         # specified in the 'retrieve_temporary_list' key
@@ -460,7 +488,9 @@ def retrieve_calculation(calculation: CalcJobNode, transport: Transport, retriev
     # Make sure that attaching the `retrieved` folder with a link is the last thing we do. This gives the biggest chance
     # of making this method idempotent. That is to say, if a runner gets interrupted during this action, it will simply
     # retry the retrieval, unless we got here and managed to link it up, in which case we move to the next task.
-    retrieved_files.add_incoming(calculation, link_type=LinkType.CREATE, link_label=calculation.link_label_retrieved)
+    retrieved_files.base.links.add_incoming(
+        calculation, link_type=LinkType.CREATE, link_label=calculation.link_label_retrieved
+    )
 
 
 def kill_calculation(calculation: CalcJobNode, transport: Transport) -> None:
@@ -498,41 +528,6 @@ def kill_calculation(calculation: CalcJobNode, transport: Transport) -> None:
             )
 
 
-def _retrieve_singlefiles(
-    job: CalcJobNode,
-    transport: Transport,
-    folder: SandboxFolder,
-    retrieve_file_list: List[Tuple[str, str, str]],
-    logger_extra: Optional[dict] = None
-):
-    """Retrieve files specified through the singlefile list mechanism."""
-    singlefile_list = []
-    for (linkname, subclassname, filename) in retrieve_file_list:
-        EXEC_LOGGER.debug(
-            '[retrieval of calc {}] Trying '
-            "to retrieve remote singlefile '{}'".format(job.pk, filename),
-            extra=logger_extra
-        )
-        localfilename = os.path.join(folder.abspath, os.path.split(filename)[1])
-        transport.get(filename, localfilename, ignore_nonexisting=True)
-        singlefile_list.append((linkname, subclassname, localfilename))
-
-    # ignore files that have not been retrieved
-    singlefile_list = [i for i in singlefile_list if os.path.exists(i[2])]
-
-    # after retrieving from the cluster, I create the objects
-    singlefiles = []
-    for (linkname, subclassname, filename) in singlefile_list:
-        cls = DataFactory(subclassname)
-        singlefile = cls(file=filename)
-        singlefile.add_incoming(job, link_type=LinkType.CREATE, link_label=linkname)
-        singlefiles.append(singlefile)
-
-    for fil in singlefiles:
-        EXEC_LOGGER.debug(f'[retrieval of calc {job.pk}] Storing retrieved_singlefile={fil.pk}', extra=logger_extra)
-        fil.store()
-
-
 def retrieve_files_from_list(
     calculation: CalcJobNode, transport: Transport, folder: str, retrieve_list: List[Union[str, Tuple[str, str, int],
                                                                                            list]]
