@@ -15,6 +15,8 @@ import collections.abc
 
 from .array import ArrayData
 
+__all__ = ('TrajectoryData',)
+
 
 class TrajectoryData(ArrayData):
     """
@@ -37,7 +39,7 @@ class TrajectoryData(ArrayData):
 
         if not isinstance(symbols, collections.abc.Iterable):
             raise TypeError('TrajectoryData.symbols must be of type list')
-        if any([not isinstance(i, str) for i in symbols]):
+        if any(not isinstance(i, str) for i in symbols):
             raise TypeError('TrajectoryData.symbols must be a 1d list of strings')
         if not isinstance(positions, numpy.ndarray) or positions.dtype != float:
             raise TypeError('TrajectoryData.positions must be a numpy array of floats')
@@ -136,7 +138,7 @@ class TrajectoryData(ArrayData):
 
         self._internal_validate(stepids, cells, symbols, positions, times, velocities)
         # set symbols as attribute for easier querying
-        self.set_attribute('symbols', list(symbols))
+        self.base.attributes.set('symbols', list(symbols))
         self.set_array('positions', positions)
         if stepids is not None:  # use input stepids
             self.set_array('steps', stepids)
@@ -269,7 +271,7 @@ class TrajectoryData(ArrayData):
 
         :raises KeyError: if the trajectory has not been set yet.
         """
-        return self.get_attribute('symbols')
+        return self.base.attributes.get('symbols')
 
     def get_positions(self):
         """
@@ -375,7 +377,7 @@ class TrajectoryData(ArrayData):
           meaning that the strings in the ``symbols`` array must be valid
           chemical symbols.
         """
-        from aiida.orm.nodes.data.structure import StructureData, Kind, Site
+        from aiida.orm.nodes.data.structure import Kind, Site, StructureData
 
         # ignore step, time, and velocities
         _, _, cell, symbols, positions, _ = self.get_step_data(index)
@@ -438,7 +440,7 @@ class TrajectoryData(ArrayData):
         for idx in indices:
             return_string += f'PRIMVEC {idx + 1}\n'
             for cell_vector in cells[idx]:
-                return_string += ' '.join(['{:18.5f}'.format(i) for i in cell_vector])
+                return_string += ' '.join([f'{i:18.5f}' for i in cell_vector])
                 return_string += '\n'
             return_string += f'PRIMCOORD {idx + 1}\n'
             return_string += f'{nat} 1\n'
@@ -454,9 +456,8 @@ class TrajectoryData(ArrayData):
         """
         Write the given trajectory to a string of format CIF.
         """
-        from aiida.orm.nodes.data.cif \
-            import ase_loops, cif_from_ase, pycifrw_from_cif
         from aiida.common.utils import Capturing
+        from aiida.orm.nodes.data.cif import ase_loops, cif_from_ase, pycifrw_from_cif
 
         cif = ''
         indices = list(range(self.numsteps))
@@ -523,9 +524,10 @@ class TrajectoryData(ArrayData):
             t.importfile('some-calc/AIIDA-PROJECT-pos-1.xyz', 'xyz_pos')
         """
 
+        from numpy import array
+
         from aiida.common.exceptions import ValidationError
         from aiida.tools.data.structure import xyz_parser_iterator
-        from numpy import array
 
         numsteps = self.numsteps
         if numsteps == 0:
@@ -557,9 +559,10 @@ class TrajectoryData(ArrayData):
             :py:meth:`._parse_xyz_pos`
         """
 
+        from numpy import array
+
         from aiida.common.exceptions import ValidationError
         from aiida.tools.data.structure import xyz_parser_iterator
-        from numpy import array
 
         numsteps = self.numsteps
         if numsteps == 0:
@@ -599,7 +602,6 @@ class TrajectoryData(ArrayData):
         :param bool dont_block: If True, interpreter is not blocked when figure is displayed.
         """
         from ase.data import atomic_numbers
-        from aiida.common.exceptions import InputValidationError
 
         # Reading the arrays I need:
         positions = self.get_positions()
@@ -608,12 +610,12 @@ class TrajectoryData(ArrayData):
 
         # Try to get the units.
         try:
-            positions_unit = self.get_attribute('units|positions')
-        except KeyError:
+            positions_unit = self.base.attributes.get('units|positions')
+        except AttributeError:
             positions_unit = 'A'
         try:
-            times_unit = self.get_attribute('units|times')
-        except KeyError:
+            times_unit = self.base.attributes.get('units|times')
+        except AttributeError:
             times_unit = 'ps'
 
         # Getting the keyword input
@@ -632,9 +634,9 @@ class TrajectoryData(ArrayData):
         elif colors == 'cpk':
             from ase.data.colors import cpk_colors as colors
         else:
-            raise InputValidationError(f'Unknown color spec {colors}')
+            raise ValueError(f'Unknown color spec {colors}')
         if kwargs:
-            raise InputValidationError(f'Unrecognized keyword {kwargs.keys()}')
+            raise ValueError(f'Unrecognized keyword {kwargs.keys()}')
 
         if element_list is None:
             # If not all elements are allowed
@@ -691,8 +693,8 @@ class TrajectoryData(ArrayData):
                 'and requires that you already installed the python numpy '
                 'package, as well as the vtk package'
             )
-        from ase.data.colors import jmol_colors
         from ase.data import atomic_numbers
+        from ase.data.colors import jmol_colors
 
         # pylint: disable=invalid-name
 
@@ -703,7 +705,7 @@ class TrajectoryData(ArrayData):
             point given results in the point being given as a multiples of lattice vectors
             Than take the integer of the rows to find how many times you have to shift
             the point back"""
-            invcell = np.matrix(cell).T.I
+            invcell = np.matrix(cell).T.I  # pylint: disable=no-member
             # point in crystal coordinates
             points_in_crystal = np.dot(invcell, point).tolist()[0]
             #point collapsed into unit cell
@@ -729,10 +731,10 @@ class TrajectoryData(ArrayData):
         positions = self.get_positions()[minindex:maxindex:stepsize]
 
         try:
-            if self.get_attribute('units|positions') in ('bohr', 'atomic'):
+            if self.base.attributes.get('units|positions') in ('bohr', 'atomic'):
                 bohr_to_ang = 0.52917720859
                 positions *= bohr_to_ang
-        except KeyError:
+        except AttributeError:
             pass
 
         symbols = self.symbols
