@@ -8,20 +8,46 @@
 # For further information please visit http://www.aiida.net               #
 ###########################################################################
 """Utilities for dealing with links between nodes."""
-from collections import namedtuple, OrderedDict
+from collections import OrderedDict
 from collections.abc import Mapping
+from typing import TYPE_CHECKING, Generator, Iterator, List, NamedTuple, Optional
 
 from aiida.common import exceptions
 from aiida.common.lang import type_check
 
+if TYPE_CHECKING:
+    from aiida.common.links import LinkType
+    from aiida.orm import Node
+    from aiida.orm.implementation.storage_backend import StorageBackend
+
 __all__ = ('LinkPair', 'LinkTriple', 'LinkManager', 'validate_link')
 
-LinkPair = namedtuple('LinkPair', ['link_type', 'link_label'])
-LinkTriple = namedtuple('LinkTriple', ['node', 'link_type', 'link_label'])
-LinkQuadruple = namedtuple('LinkQuadruple', ['source_id', 'target_id', 'link_type', 'link_label'])
+
+class LinkPair(NamedTuple):
+    link_type: 'LinkType'
+    link_label: str
 
 
-def link_triple_exists(source, target, link_type, link_label):
+class LinkTriple(NamedTuple):
+    node: 'Node'
+    link_type: 'LinkType'
+    link_label: str
+
+
+class LinkQuadruple(NamedTuple):
+    source_id: int
+    target_id: int
+    link_type: 'LinkType'
+    link_label: str
+
+
+def link_triple_exists(
+    source: 'Node',
+    target: 'Node',
+    link_type: 'LinkType',
+    link_label: str,
+    backend: Optional['StorageBackend'] = None
+) -> bool:
     """Return whether a link with the given type and label exists between the given source and target node.
 
     :param source: node from which the link is outgoing
@@ -32,8 +58,10 @@ def link_triple_exists(source, target, link_type, link_label):
     """
     from aiida.orm import Node, QueryBuilder
 
+    target_links_cache = target.base.links.incoming_cache
+
     # First check if the triple exist in the cache, in the case of an unstored target node
-    if target._incoming_cache and LinkTriple(source, link_type, link_label) in target._incoming_cache:  # pylint: disable=protected-access
+    if target_links_cache and LinkTriple(source, link_type, link_label) in target_links_cache:
         return True
 
     # If either node is unstored (i.e. does not have a pk), the link cannot exist in the database, so no need to check
@@ -42,15 +70,21 @@ def link_triple_exists(source, target, link_type, link_label):
 
     # Here we have two stored nodes, so we need to check if the same link already exists in the database.
     # Finding just a single match is sufficient so we can use the `limit` clause for efficiency
-    builder = QueryBuilder()
-    builder.append(Node, filters={'id': source.id}, project=['id'])
-    builder.append(Node, filters={'id': target.id}, edge_filters={'type': link_type.value, 'label': link_label})
+    builder = QueryBuilder(backend=backend)
+    builder.append(Node, filters={'id': source.pk}, project=['id'])
+    builder.append(Node, filters={'id': target.pk}, edge_filters={'type': link_type.value, 'label': link_label})
     builder.limit(1)
 
     return builder.count() != 0
 
 
-def validate_link(source, target, link_type, link_label):
+def validate_link(
+    source: 'Node',
+    target: 'Node',
+    link_type: 'LinkType',
+    link_label: str,
+    backend: Optional['StorageBackend'] = None
+) -> None:
     """
     Validate adding a link of the given type and label from a given node to ourself.
 
@@ -114,7 +148,7 @@ def validate_link(source, target, link_type, link_label):
     """
     # yapf: disable
     from aiida.common.links import LinkType, validate_link_label
-    from aiida.orm import Node, Data, CalculationNode, WorkflowNode
+    from aiida.orm import CalculationNode, Data, Node, WorkflowNode
 
     type_check(link_type, LinkType, f'link_type should be a LinkType enum but got: {type(link_type)}')
     type_check(source, Node, f'source should be a `Node` but got: {type(source)}')
@@ -153,14 +187,14 @@ def validate_link(source, target, link_type, link_label):
     if outdegree == 'unique_triple' or indegree == 'unique_triple':
         # For a `unique_triple` degree we just have to check if an identical triple already exist, either in the cache
         # or stored, in which case, the new proposed link is a duplicate and thus illegal
-        duplicate_link_triple = link_triple_exists(source, target, link_type, link_label)
+        duplicate_link_triple = link_triple_exists(source, target, link_type, link_label, backend)
 
     # If the outdegree is `unique` there cannot already be any other outgoing link of that type
-    if outdegree == 'unique' and source.get_outgoing(link_type=link_type, only_uuid=True).all():
+    if outdegree == 'unique' and source.base.links.get_outgoing(link_type=link_type, only_uuid=True).all():
         raise ValueError(f'node<{source.uuid}> already has an outgoing {link_type} link')
 
     # If the outdegree is `unique_pair`, then the link labels for outgoing links of this type should be unique
-    elif outdegree == 'unique_pair' and source.get_outgoing(
+    elif outdegree == 'unique_pair' and source.base.links.get_outgoing(
             link_type=link_type, only_uuid=True, link_label_filter=link_label).all():
         raise ValueError(f'node<{source.uuid}> already has an outgoing {link_type} link with label "{link_label}"')
 
@@ -170,11 +204,11 @@ def validate_link(source, target, link_type, link_label):
             source.uuid, link_type, link_label, target.uuid))
 
     # If the indegree is `unique` there cannot already be any other incoming links of that type
-    if indegree == 'unique' and target.get_incoming(link_type=link_type, only_uuid=True).all():
+    if indegree == 'unique' and target.base.links.get_incoming(link_type=link_type, only_uuid=True).all():
         raise ValueError(f'node<{target.uuid}> already has an incoming {link_type} link')
 
     # If the indegree is `unique_pair`, then the link labels for incoming links of this type should be unique
-    elif indegree == 'unique_pair' and target.get_incoming(
+    elif indegree == 'unique_pair' and target.base.links.get_incoming(
             link_type=link_type, link_label_filter=link_label, only_uuid=True).all():
         raise ValueError(f'node<{target.uuid}> already has an incoming {link_type} link with label "{link_label}"')
 
@@ -199,18 +233,18 @@ class LinkManager:
     incoming nodes or link labels, respectively.
     """
 
-    def __init__(self, link_triples):
+    def __init__(self, link_triples: List[LinkTriple]):
         """Initialise the collection."""
         self.link_triples = link_triples
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[LinkTriple]:
         """Return an iterator of LinkTriple instances.
 
         :return: iterator of LinkTriple instances
         """
         return iter(self.link_triples)
 
-    def __next__(self):
+    def __next__(self) -> Generator[LinkTriple, None, None]:
         """Return the next element in the iterator.
 
         :return: LinkTriple
@@ -221,14 +255,14 @@ class LinkManager:
     def __bool__(self):
         return bool(len(self.link_triples))
 
-    def next(self):
+    def next(self) -> Generator[LinkTriple, None, None]:
         """Return the next element in the iterator.
 
         :return: LinkTriple
         """
         return self.__next__()
 
-    def one(self):
+    def one(self) -> LinkTriple:
         """Return a single entry from the iterator.
 
         If the iterator contains no or more than one entry, an exception will be raised
@@ -242,7 +276,7 @@ class LinkManager:
 
         raise ValueError('no entries found')
 
-    def first(self):
+    def first(self) -> Optional[LinkTriple]:
         """Return the first entry from the iterator.
 
         :return: LinkTriple instance or None if no entries were matched
@@ -252,35 +286,35 @@ class LinkManager:
 
         return None
 
-    def all(self):
+    def all(self) -> List[LinkTriple]:
         """Return all entries from the list.
 
         :return: list of LinkTriple instances
         """
         return self.link_triples
 
-    def all_nodes(self):
+    def all_nodes(self) -> List['Node']:
         """Return a list of all nodes.
 
         :return: list of nodes
         """
         return [entry.node for entry in self.all()]
 
-    def all_link_pairs(self):
+    def all_link_pairs(self) -> List[LinkPair]:
         """Return a list of all link pairs.
 
         :return: list of LinkPair instances
         """
         return [LinkPair(entry.link_type, entry.link_label) for entry in self.all()]
 
-    def all_link_labels(self):
+    def all_link_labels(self) -> List[str]:
         """Return a list of all link labels.
 
         :return: list of link labels
         """
         return [entry.link_label for entry in self.all()]
 
-    def get_node_by_label(self, label):
+    def get_node_by_label(self, label: str) -> 'Node':
         """Return the node from list for given label.
 
         :return: node that corresponds to the given label
@@ -313,7 +347,7 @@ class LinkManager:
         """
         from aiida.engine.processes.ports import PORT_NAMESPACE_SEPARATOR
 
-        nested = {}
+        nested: dict = {}
 
         for entry in self.link_triples:
 
